#include "drawarea.h"
#include <QPainter>

DrawArea::DrawArea(QWidget *parent):
    QWidget(parent),
    xmin(-8.),
    xmax(8.),
    ymin(-6.),
    ymax(6.),
    xcoeff(800./(xmax - xmin)),
    ycoeff(600./(ymax - ymin)),
    center(),
    radius(0.),
    triangleComputed(false)
{

}

void DrawArea::paintEvent(QPaintEvent *) {
    QPainter painter(this);
    painter.fillRect(
        QRect(0, 0, width(), height()),
        QBrush(Qt::lightGray)
    );

    drawCoordGrid(&painter);

    // Draw points
    R2Vector dx(0.2, 0);
    R2Vector dy(0., 0.2);
    painter.setPen(Qt::red);
    for (size_t i = 0; i < points.size(); ++i) {
        painter.drawLine(
           map(points[i] - dx),
           map(points[i] + dx)
        );
        painter.drawLine(
           map(points[i] - dy),
           map(points[i] + dy)
        );
    }

    if (triangleComputed) {
        drawTriangle(&painter);
    }
}

void DrawArea::drawCoordGrid(QPainter *painter) {
    painter->setPen(Qt::gray);
    int i = (int) xmin;
    while (i <= (int) xmax) {
        painter->drawLine(
            map(R2Point(i, ymin)),
            map(R2Point(i, ymax))
        );
        ++i;
    }

    i = (int) ymin;
    while (i <= (int) ymax) {
        painter->drawLine(
            map(R2Point(xmin, i)),
            map(R2Point(xmax, i))
        );
        ++i;
    }

    // Coord. axis
    painter->setPen(Qt::black);
    painter->drawLine(
        map(R2Point(xmin, 0.)),
        map(R2Point(xmax, 0.))
    );
    painter->drawLine(
        map(R2Point(0., ymin)),
        map(R2Point(0., ymax))
    );
}

void DrawArea::mousePressEvent(QMouseEvent *e) {
    QPointF p = e->localPos();
    R2Point t = invMap(p);
    points.push_back(t);
    update();
}

void DrawArea::clear() {
    points.clear();
    triangleComputed = false;
    update();
}

void DrawArea::onDraw() {
    if (points.size() < 3)
        return;
    for (int i = 0; i < 3; ++i) {
        // Ищем бисектриссу i-го угла
        int next = (i + 1)%3;
        int prev = (i - 1)%3;
        if (prev < 0)
            prev += 3;
        R2Vector v1 = points[next] -
            points[i];
        v1.normalize();
        R2Vector v2 = points[prev] -
            points[i];
        v2.normalize();
        R2Vector b = v1 + v2;
        intersectStraightLines(
            points[i], b,
            points[next],
            points[prev] - points[next],
            bis[i]
        );
    }

    intersectStraightLines(
        points[0], bis[0] - points[0],
        points[1], bis[1] - points[1],
        center
    );

    R2Vector n =
    (points[1] - points[0]).normal();
    n.normalize();
    radius = n*(center - points[0]);

    triangleComputed = true;
    update();
}

void DrawArea::drawTriangle(QPainter *painter) {
    painter->setRenderHints(QPainter::Antialiasing);
    QPainterPath path;
    path.moveTo(map(points[0]));
    path.lineTo(map(points[1]));
    path.lineTo(map(points[2]));
    path.closeSubpath();
    painter->setPen(Qt::blue);
    painter->drawPath(path);

    painter->setPen(Qt::darkGreen);
    for (int i = 0; i < 3; ++i) {
        painter->drawLine(
            map(points[i]),
            map(bis[i])
        );
    }

    QPointF c = map(center);
    painter->setPen(Qt::red);
    painter->drawEllipse(
        c,
        radius*xcoeff,
        radius*ycoeff
    );

    painter->setBrush(QBrush(Qt::red));
    painter->drawEllipse(c, 2, 2);
}

void DrawArea::resizeEvent(QResizeEvent *) {
    if (height() == 0 || width() == 0)
        return;

    double aspect = double(height())/
        double(width());
    double yCenter = (ymin + ymax)/2.;
    double xWidth = xmax - xmin;
    ymax = yCenter + (xWidth/2.)*aspect;
    ymin = yCenter - (xWidth/2.)*aspect;

    xcoeff = width()/(xmax - xmin);
    ycoeff = height()/(ymax - ymin);
    // update();
}
